package cn.com.started.mseureka;

import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.util.Random;
import java.util.UUID;

 /**
 * 验证码工具类
 */
public abstract class VCodeUtils {
    private static final Logger logger = LoggerFactory.getLogger(VCodeUtils.class);
    private static final String SMS_VCODES = "1234567890";
    private static final String GV_CODES = "23578AaBbCcDdEeFfGHhJjKkLlMmNnPpQRrSsTtUuVvWwXxYyZz";
    private static final Random random = new Random();

    public static String getVCodeKey() {
        return UUID.randomUUID().toString().replace("-", "");
    }

    /**
     * 创建短信验证码
     *
     * @param codeSize 验证码长度
     * @return
     */
    public static String createSMSVCode(Integer codeSize) {
        if (codeSize == null || codeSize == 0) {
            codeSize = 6;
        }

        int codesLen = SMS_VCODES.length();
        StringBuffer randomCode = new StringBuffer();
        for (int i = 0; i < codeSize; i++) {
            char randChar = SMS_VCODES.charAt(random.nextInt(codesLen));
            randomCode.append(randChar);
        }
        return randomCode.toString();
    }

    /**
     * 创建图形验证码
     *
     * @param codeSize 验证码个数
     * @return
     */
    public static String createGVCode(int codeSize) {
        int codesLen = GV_CODES.length();

        StringBuffer gvCode = new StringBuffer();
        for (int i = 0; i < codeSize; i++) {
            gvCode.append(GV_CODES.charAt(random.nextInt(codesLen - 1)));
        }
        return gvCode.toString();
    }


    /**
     * 绘制图形验证码图片
     *
     * @param vcode     验证码
     * @param imgWidth  图片宽度
     * @param imgHeight 图片高度
     * @throws Exception
     */
    public static String drawGVCodeImg(String vcode, int imgWidth, int imgHeight) throws Exception {
        String gvCodeImg = null;

        ByteArrayOutputStream output = new ByteArrayOutputStream();
        try {
            int codeSize = vcode.length();
            Random random = new Random();
            int fontSize = imgHeight - 15;

            BufferedImage image = new BufferedImage(imgWidth, imgHeight, BufferedImage.TYPE_INT_RGB);
            Graphics2D g = image.createGraphics();
            g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g.setFont(new Font("Algerian", Font.ITALIC, fontSize));
            g.setColor(Color.GRAY);// 设置边框色
            g.fillRect(0, 0, imgWidth, imgHeight);

            Color c = getRandColor(200, 250);
            g.setBackground(c);// 设置背景色
            g.fillRect(0, 2, imgWidth, imgHeight - 4);

            //绘制干扰线
            g.setColor(getRandColor(160, 200));// 设置线条的颜色
            for (int i = 0; i < 20; i++) {
                int x = random.nextInt(imgWidth - 1);
                int y = random.nextInt(imgHeight - 1);
                int xl = random.nextInt(6) + 1;
                int yl = random.nextInt(12) + 1;
                g.drawLine(x, y, x + xl + 40, y + yl + 20);
            }

            // 添加噪点
            float yawpRate = 0.25f;// 噪声率
            int area = (int) (yawpRate * imgWidth * imgHeight);
            for (int i = 0; i < area; i++) {
                int x = random.nextInt(imgWidth);
                int y = random.nextInt(imgHeight);
                int rgb = getRandColor();
                image.setRGB(x, y, rgb);
            }

            char[] chars = vcode.toCharArray();
            for (int i = 0; i < codeSize; i++) {
                AffineTransform affine = new AffineTransform();
                affine.setToRotation(Math.PI / 4 * random.nextDouble() * (random.nextBoolean() ? 1 : -1), (imgWidth / codeSize) * i + fontSize / 2, imgHeight / 2);
                g.setTransform(affine);
                g.drawChars(chars, i, 1, ((imgWidth - 10) / codeSize) * i + 5, imgHeight / 2 + fontSize / 2 - 10);
            }

            g.dispose();
            image.flush();

            ImageIO.write(image, "png", output);
            output.flush();

            gvCodeImg = Base64.encodeBase64String(output.toByteArray());
            return gvCodeImg;
        } catch (Exception e) {
            logger.error("系统生成图形验证码 {} 失败: {}", vcode, e.getMessage());
            throw e;
        } finally {
            if (null != output) {
                output.close();
            }
        }
    }

    private static Color getRandColor(int fc, int bc) {
        fc = fc > 255 ? 255 : fc;
        bc = bc > 255 ? 255 : bc;

        int r = fc + random.nextInt(bc - fc);
        int g = fc + random.nextInt(bc - fc);
        int b = fc + random.nextInt(bc - fc);
        return new Color(r, g, b);
    }

    private static int getRandColor() {
        int[] rgb = new int[3];
        for (int i = 0; i < 3; i++) {
            rgb[i] = random.nextInt(255);
        }

        int color = 0;
        for (int c : rgb) {
            color = color << 8;
            color = color | c;
        }
        return color;
    }

    public static void main(String[] args) throws Exception {
        System.out.println(getVCodeKey());
        //System.out.println(createSMSVCode(6));
        String gvCodeImg = drawGVCodeImg(createGVCode(4), 150, 50);
        System.out.println(gvCodeImg.length() + ": data:image/png;base64," + gvCodeImg);
    }
}
